"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.Build = void 0;

var _chalk = _interopRequireDefault(require("chalk"));

var _lodash = _interopRequireDefault(require("lodash"));

var _chokidar = _interopRequireDefault(require("chokidar"));

var _fs = require("fs");

var _path = require("path");

var _build = require("../lib/build");

var _logger = require("../lib/utils/logger");

var _helps = require("./helps");

var _require = require("../lib/utils/require");

var _constants = require("../lib/constants");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }

var Build =
/*#__PURE__*/
function () {
  var _ref = _asyncToGenerator(
  /*#__PURE__*/
  regeneratorRuntime.mark(function _callee2(args) {
    var dir;
    return regeneratorRuntime.wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            if (args.help) {
              (0, _helps.logBuildHelp)();
            } // Start


            dir = (0, _path.resolve)(args.path);

            if (args.watch) {
              console.log(_chalk.default.green("\uD83E\uDD86  Watching decks in ".concat((0, _path.basename)(dir), "...")));
            } else {
              console.log(_chalk.default.green("\uD83E\uDD86  Building decks in ".concat((0, _path.basename)(dir), "...")));
            } // Check Dependencies


            if (!(0, _fs.existsSync)(dir)) {
              (0, _logger.printAndExit)(_chalk.default.red("No such directory ".concat((0, _path.basename)(dir), ".")));
            }

            _constants.REQUIRED_SUBDIRS.forEach(function (subdir) {
              if (!(0, _fs.existsSync)((0, _path.join)(dir, subdir))) {
                (0, _logger.printAndExit)(_chalk.default.red("Couldn't find ".concat(subdir, ". Exiting.")));
              }
            }); // Build


            _context2.prev = 5;
            _context2.next = 8;
            return (0, _build.build)(dir);

          case 8:
            _context2.next = 14;
            break;

          case 10:
            _context2.prev = 10;
            _context2.t0 = _context2["catch"](5);
            console.error(_context2.t0);
            process.exit(1);

          case 14:
            if (!args.watch) {
              _context2.next = 17;
              break;
            }

            _context2.next = 17;
            return new Promise(function (res, rej) {
              var watcher = _chokidar.default.watch([(0, _path.resolve)("./".concat(_constants.TEMPLATE_FOLDER, "/")), (0, _path.resolve)("./".concat(_constants.DATA_FOLDER, "/"))], {
                ignored: ["node_modules", /(^|[\/\\])\../],
                ignoreInitial: true,
                persistent: true
              }); // Add event listeners.


              var rebuild =
              /*#__PURE__*/
              function () {
                var _ref2 = _asyncToGenerator(
                /*#__PURE__*/
                regeneratorRuntime.mark(function _callee(p) {
                  var pathRegex;
                  return regeneratorRuntime.wrap(function _callee$(_context) {
                    while (1) {
                      switch (_context.prev = _context.next) {
                        case 0:
                          // TODO: This is a weird constant
                          pathRegex = new RegExp("".concat((0, _path.resolve)("./".concat(_constants.TMP_FOLDER)), ".*"));
                          (0, _require.clearCache)(pathRegex);
                          console.log(_chalk.default.green("\n\uD83D\uDEE0  Rebuilding ".concat((0, _path.basename)(dir))));
                          _context.next = 5;
                          return (0, _build.build)(dir);

                        case 5:
                        case "end":
                          return _context.stop();
                      }
                    }
                  }, _callee, this);
                }));

                return function rebuild(_x2) {
                  return _ref2.apply(this, arguments);
                };
              }(); // Handle new and modified files


              watcher.on("add", rebuild);
              watcher.on("change", rebuild); // Handle removing files

              watcher.on("unlink", function (p) {
                watcher.unwatch(p);
                rebuild(p);
              }); // Handle errors

              watcher.on("error", function (error) {
                console.log(_chalk.default.red(error));
                rej(error);
              }); // Handle interrupts

              process.on("SIGINT", function () {
                watcher.close();
                res();
              });
              process.on("SIGTERM", function () {
                watcher.close();
                res();
              });
            });

          case 17:
            // Done
            (0, _logger.printAndExit)(_chalk.default.green("✨ Build complete!"));

          case 18:
          case "end":
            return _context2.stop();
        }
      }
    }, _callee2, this, [[5, 10]]);
  }));

  return function Build(_x) {
    return _ref.apply(this, arguments);
  };
}();

exports.Build = Build;