"use strict";

require("babel-polyfill");

var _chalk = _interopRequireDefault(require("chalk"));

var _lodash = _interopRequireDefault(require("lodash"));

var _minimist = _interopRequireDefault(require("minimist"));

var _build = require("./cmd/build.js");

var _logger = require("./lib/utils/logger");

var _init = require("./cmd/init.js");

var _helps = require("./cmd/helps.js");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

process.on("SIGINT", function () {
  process.exit(0);
});
process.on("SIGTERM", function () {
  process.exit(0);
});
var defaultCmd = "build";
var commands = {
  init: _init.Init,
  build: _build.Build
};
var argv = (0, _minimist.default)(process.argv.slice(2), {
  alias: {
    h: "help",
    p: "path",
    P: "proof",
    w: "watch",
    v: "verbose"
  },
  boolean: ["h", "P", "w", "v"],
  string: ["p"],
  default: {
    path: "./",
    proof: false,
    watch: false,
    verbose: false
  }
});
var cmd = argv._[0];

var availableCmds = _lodash.default.keys(commands);

if (argv.proof) {
  process.env.PROOF = true;
}

if (argv.verbose) {
  process.env.VERBOSE = true;
}

if (!cmd && argv.help) {
  (0, _helps.logDefaultHelp)(availableCmds);
}

if (!cmd) {
  var promise = commands[defaultCmd](argv);

  if (promise) {
    promise.then(function () {
      return process.exit(0);
    }).catch(function (e) {
      return console.error(_chalk.default.red(e.stack));
    });
  }
} else if (cmd && _lodash.default.has(commands, cmd)) {
  var _promise = commands[cmd](argv);

  if (_promise) {
    _promise.then(function () {
      return process.exit(0);
    }).catch(function (e) {
      console.error(_chalk.default.red(e.stack));
    });
  }
} else {
  console.warn(_chalk.default.yellow("Warning: Unknown Command: ".concat(cmd)));
  (0, _helps.logDefaultHelp)(availableCmds);
}