"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.formatCards = void 0;

var _convertSvgToPng = require("convert-svg-to-png");

var _mississippi = _interopRequireDefault(require("mississippi"));

var _path = _interopRequireDefault(require("path"));

var _rimraf = _interopRequireDefault(require("rimraf"));

var _nullStream = require("./streams/nullStream");

var _cropStream = require("./streams/cropStream");

var _pdfStream = require("./streams/pdfStream");

var _pngStream = require("./streams/pngStream");

var _svgStream = require("./streams/svgStream");

var _fsp = _interopRequireDefault(require("./utils/fsp"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }

var formatCards =
/*#__PURE__*/
function () {
  var _ref = _asyncToGenerator(
  /*#__PURE__*/
  regeneratorRuntime.mark(function _callee(projectRoot, config, data, renderings) {
    var deckKey;
    return regeneratorRuntime.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            _context.t0 = regeneratorRuntime.keys(config);

          case 1:
            if ((_context.t1 = _context.t0()).done) {
              _context.next = 7;
              break;
            }

            deckKey = _context.t1.value;
            _context.next = 5;
            return formatDeck(projectRoot, config[deckKey], renderings[deckKey], deckKey);

          case 5:
            _context.next = 1;
            break;

          case 7:
          case "end":
            return _context.stop();
        }
      }
    }, _callee, this);
  }));

  return function formatCards(_x, _x2, _x3, _x4) {
    return _ref.apply(this, arguments);
  };
}();

exports.formatCards = formatCards;

var formatDeck =
/*#__PURE__*/
function () {
  var _ref2 = _asyncToGenerator(
  /*#__PURE__*/
  regeneratorRuntime.mark(function _callee3(projectRoot, config, renderings, deckKey) {
    var output, svg, png, pdf, converter, size, renderingStream;
    return regeneratorRuntime.wrap(function _callee3$(_context3) {
      while (1) {
        switch (_context3.prev = _context3.next) {
          case 0:
            _context3.next = 2;
            return deckFolder(projectRoot, deckKey);

          case 2:
            output = _context3.sent;
            svg = config.format.includes("svg");
            png = config.format.includes("png");
            pdf = config.format.includes("pdf");
            converter = pdf || png ? (0, _convertSvgToPng.createConverter)() : null;
            size = renderings.length;
            renderingStream = _mississippi.default.from.obj(renderings);
            _context3.next = 11;
            return new Promise(function (res, rej) {
              process.on("SIGINT", rej);

              var finalize =
              /*#__PURE__*/
              function () {
                var _ref3 = _asyncToGenerator(
                /*#__PURE__*/
                regeneratorRuntime.mark(function _callee2(err) {
                  return regeneratorRuntime.wrap(function _callee2$(_context2) {
                    while (1) {
                      switch (_context2.prev = _context2.next) {
                        case 0:
                          if (converter) converter.destroy();
                          if (err) rej(err);
                          res();

                        case 3:
                        case "end":
                          return _context2.stop();
                      }
                    }
                  }, _callee2, this);
                }));

                return function finalize(_x9) {
                  return _ref3.apply(this, arguments);
                };
              }();

              if (svg) {
                renderingStream = renderingStream.pipe((0, _svgStream.svgStream)({
                  output: output,
                  size: size
                }));
              }

              if ((png || pdf) && converter) {
                renderingStream = renderingStream.pipe((0, _pngStream.pngStream)({
                  output: output,
                  converter: converter,
                  size: size
                }));
              }

              if (pdf) {
                renderingStream = renderingStream.pipe((0, _cropStream.cropStream)({
                  config: config,
                  size: size
                })).pipe((0, _pdfStream.pdfStream)({
                  output: output,
                  config: config
                }));
              }

              renderingStream.pipe((0, _nullStream.nullStream)()).on("error", finalize).on("end", finalize).on("finish", finalize);
            });

          case 11:
            return _context3.abrupt("return", _context3.sent);

          case 12:
          case "end":
            return _context3.stop();
        }
      }
    }, _callee3, this);
  }));

  return function formatDeck(_x5, _x6, _x7, _x8) {
    return _ref2.apply(this, arguments);
  };
}();

var deckFolder =
/*#__PURE__*/
function () {
  var _ref4 = _asyncToGenerator(
  /*#__PURE__*/
  regeneratorRuntime.mark(function _callee4(projectRoot, deckKey) {
    var output, folder;
    return regeneratorRuntime.wrap(function _callee4$(_context4) {
      while (1) {
        switch (_context4.prev = _context4.next) {
          case 0:
            output = _path.default.join(projectRoot, "output");
            folder = _path.default.join(output, deckKey);
            _context4.next = 4;
            return _fsp.default.exists(output);

          case 4:
            if (_context4.sent) {
              _context4.next = 7;
              break;
            }

            _context4.next = 7;
            return _fsp.default.mkdir(output);

          case 7:
            _context4.next = 9;
            return _fsp.default.exists(folder);

          case 9:
            if (!_context4.sent) {
              _context4.next = 11;
              break;
            }

            _rimraf.default.sync(folder);

          case 11:
            _context4.next = 13;
            return _fsp.default.exists(folder);

          case 13:
            if (_context4.sent) {
              _context4.next = 16;
              break;
            }

            _context4.next = 16;
            return _fsp.default.mkdir(folder);

          case 16:
            return _context4.abrupt("return", folder);

          case 17:
          case "end":
            return _context4.stop();
        }
      }
    }, _callee4, this);
  }));

  return function deckFolder(_x10, _x11) {
    return _ref4.apply(this, arguments);
  };
}();