"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.formatPdf = void 0;

var _fs = _interopRequireDefault(require("fs"));

var _lodash = _interopRequireDefault(require("lodash"));

var _path = _interopRequireDefault(require("path"));

var _pdfkit = _interopRequireDefault(require("pdfkit"));

var _progressBar = require("../utils/progressBar");

var _units = require("../utils/units");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }

var formatPdf =
/*#__PURE__*/
function () {
  var _ref = _asyncToGenerator(
  /*#__PURE__*/
  regeneratorRuntime.mark(function _callee(pngBuffers, out, config) {
    var _config$pdf, _config$pdf$layout, layout, _config$pdf$size, size, _config$pdf$margin, margin, bleed, docConfig, doc, cardWidthPx, cardHeightPx, marginPts, docWidth, docHeight, cardWidthPts, cardHeightPts, pageWidth, pageHeight, cardsPerRow, rowsPerPage, rows, pages, trimLines;

    return regeneratorRuntime.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            _config$pdf = config.pdf, _config$pdf$layout = _config$pdf.layout, layout = _config$pdf$layout === void 0 ? "landscape" : _config$pdf$layout, _config$pdf$size = _config$pdf.size, size = _config$pdf$size === void 0 ? "letter" : _config$pdf$size, _config$pdf$margin = _config$pdf.margin, margin = _config$pdf$margin === void 0 ? 0.5 : _config$pdf$margin, bleed = _config$pdf.bleed;
            docConfig = {
              layout: layout,
              size: size
            };
            doc = new _pdfkit.default(docConfig); // Get and crop images, if needed

            cardWidthPx = config.width;
            cardHeightPx = config.height;

            if (bleed) {
              cardWidthPx = cardWidthPx - bleed * 2;
              cardHeightPx = cardHeightPx - bleed * 2;
            } // Calculate all layout values


            marginPts = (0, _units.insToPts)(margin);
            docWidth = doc.page.width;
            docHeight = doc.page.height;
            cardWidthPts = (0, _units.pxToPts)(cardWidthPx);
            cardHeightPts = (0, _units.pxToPts)(cardHeightPx);
            pageWidth = docWidth - marginPts * 2; // width less margin

            pageHeight = docHeight - marginPts * 2; // width less margin

            cardsPerRow = Math.floor(pageWidth / cardWidthPts);
            rowsPerPage = Math.floor(pageHeight / cardHeightPts);
            rows = _lodash.default.chunk(pngBuffers, cardsPerRow);
            pages = _lodash.default.chunk(rows, rowsPerPage);
            trimLines = config.pdf.trim_lines; // Do layout

            return _context.abrupt("return", new Promise(function (res, err) {
              var outPath = _path.default.join(out, "out.pdf");

              var pdfStream = _fs.default.createWriteStream(outPath);

              var pdfProgress = (0, _progressBar.progressBar)("PDF", pngBuffers.length);

              try {
                doc.pipe(pdfStream);
                var cardNum = 1; // Iterate Pages

                _lodash.default.forEach(pages, function (rows, i) {
                  if (i > 0) doc.addPage(); // Iterate Rows

                  _lodash.default.forEach(rows, function (row, j) {
                    var y = j * cardHeightPts + marginPts;

                    if (trimLines) {
                      drawTrimLine(doc, 0, y, docWidth, y);
                    } // Iterate Cards


                    _lodash.default.forEach(row, function (card, k) {
                      var x = k * cardWidthPts + marginPts;

                      if (trimLines) {
                        drawTrimLine(doc, x, 0, x, docHeight);
                      }

                      doc.image(card, x, y, {
                        width: cardWidthPts,
                        height: cardHeightPts
                      });
                      pdfProgress.tick(); // Last Card in Row

                      if (k + 1 === row.length) {
                        var finalX = x + cardWidthPts;

                        if (trimLines) {
                          drawTrimLine(doc, finalX, 0, finalX, docHeight);
                        }
                      }
                    }); // Last Row on Page


                    if (j + 1 === rows.length) {
                      var finalY = y + cardHeightPts;

                      if (trimLines) {
                        drawTrimLine(doc, 0, finalY, docWidth, finalY);
                      }
                    }
                  });
                });

                doc.end();
              } catch (e) {
                err(e);
              }

              pdfStream.addListener("finish", res);
            }));

          case 19:
          case "end":
            return _context.stop();
        }
      }
    }, _callee, this);
  }));

  return function formatPdf(_x, _x2, _x3) {
    return _ref.apply(this, arguments);
  };
}();

exports.formatPdf = formatPdf;

var drawTrimLine = function drawTrimLine(doc, startX, startY, endX, endY) {
  doc.moveTo(startX, startY).lineWidth(0.5).lineTo(endX, endY).dash(5).stroke("#ccc");
};