"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.renderTemplates = void 0;

var _path = _interopRequireDefault(require("path"));

var _react = _interopRequireDefault(require("react"));

var _server = require("react-dom/server");

var _rimraf = _interopRequireDefault(require("rimraf"));

var _DeckContext = require("../../components/DeckContext");

var _fsp = _interopRequireDefault(require("./utils/fsp"));

var _transform = require("./utils/transform");

var _constants = require("./constants");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }

var renderTemplates =
/*#__PURE__*/
function () {
  var _ref = _asyncToGenerator(
  /*#__PURE__*/
  regeneratorRuntime.mark(function _callee(projectRoot, config, data) {
    var renderings, templatesPath, tmpPath, deckKey;
    return regeneratorRuntime.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            renderings = {};
            templatesPath = _path.default.join(projectRoot, _constants.TEMPLATE_FOLDER);
            tmpPath = _path.default.join(projectRoot, _constants.TMP_FOLDER);
            _context.next = 5;
            return transpileTemplates(templatesPath, tmpPath);

          case 5:
            _context.t0 = regeneratorRuntime.keys(config);

          case 6:
            if ((_context.t1 = _context.t0()).done) {
              _context.next = 13;
              break;
            }

            deckKey = _context.t1.value;
            _context.next = 10;
            return renderTemplate(tmpPath, config[deckKey], data[deckKey]);

          case 10:
            renderings[deckKey] = _context.sent;
            _context.next = 6;
            break;

          case 13:
            _rimraf.default.sync(tmpPath);

            return _context.abrupt("return", renderings);

          case 15:
          case "end":
            return _context.stop();
        }
      }
    }, _callee, this);
  }));

  return function renderTemplates(_x, _x2, _x3) {
    return _ref.apply(this, arguments);
  };
}();

exports.renderTemplates = renderTemplates;

var transpileTemplates =
/*#__PURE__*/
function () {
  var _ref2 = _asyncToGenerator(
  /*#__PURE__*/
  regeneratorRuntime.mark(function _callee2(projectRoot, tmpPath) {
    var babel;
    return regeneratorRuntime.wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            babel = {
              presets: ["@babel/preset-env", "@babel/preset-react"]
            };
            _context2.prev = 1;
            _context2.next = 4;
            return _fsp.default.readJson(_path.default.join(projectRoot, ".babelrc"));

          case 4:
            babel = _context2.sent;
            _context2.next = 9;
            break;

          case 7:
            _context2.prev = 7;
            _context2.t0 = _context2["catch"](1);

          case 9:
            _context2.next = 11;
            return (0, _transform.transformDir)(projectRoot, tmpPath, {
              babel: babel
            });

          case 11:
          case "end":
            return _context2.stop();
        }
      }
    }, _callee2, this, [[1, 7]]);
  }));

  return function transpileTemplates(_x4, _x5) {
    return _ref2.apply(this, arguments);
  };
}();

var loadTemplate = function loadTemplate(filePath) {
  return require(filePath).default;
};

var renderTemplate =
/*#__PURE__*/
function () {
  var _ref3 = _asyncToGenerator(
  /*#__PURE__*/
  regeneratorRuntime.mark(function _callee3(templatesPath, config, data) {
    var templatePath, Card, renderings, rowIdx, row, dom, svg;
    return regeneratorRuntime.wrap(function _callee3$(_context3) {
      while (1) {
        switch (_context3.prev = _context3.next) {
          case 0:
            templatePath = _path.default.resolve(_path.default.join(templatesPath, config.templateFront));
            Card = loadTemplate(templatePath);
            renderings = [];

            for (rowIdx = 0; rowIdx < data.length; rowIdx++) {
              row = data[rowIdx];
              dom = _react.default.createElement("svg", {
                xmlns: "http://www.w3.org/2000/svg"
              }, _react.default.createElement("g", null, _react.default.createElement("rect", {
                x: "0",
                y: "0",
                width: config.width,
                height: config.height,
                strokeWidth: "0",
                fill: "none"
              })), _react.default.createElement(_DeckContext.DeckProvider, {
                value: config
              }, _react.default.createElement(Card, _extends({}, row, {
                config: config,
                deck: data,
                cardIndex: rowIdx
              }))));
              svg = (0, _server.renderToStaticMarkup)(dom);
              renderings.push({
                svg: svg,
                width: config.width,
                height: config.height
              });
            }

            return _context3.abrupt("return", renderings);

          case 5:
          case "end":
            return _context3.stop();
        }
      }
    }, _callee3, this);
  }));

  return function renderTemplate(_x6, _x7, _x8) {
    return _ref3.apply(this, arguments);
  };
}();