"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.Flexbox = void 0;

var _react = _interopRequireDefault(require("react"));

var _yogaLayout = _interopRequireWildcard(require("yoga-layout"));

var _propTypes = require("../lib/propTypes");

var _yoga = require("../lib/yoga");

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = Object.defineProperty && Object.getOwnPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : {}; if (desc.get || desc.set) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _objectWithoutProperties(source, excluded) { if (source == null) return {}; var target = _objectWithoutPropertiesLoose(source, excluded); var key, i; if (Object.getOwnPropertySymbols) { var sourceSymbolKeys = Object.getOwnPropertySymbols(source); for (i = 0; i < sourceSymbolKeys.length; i++) { key = sourceSymbolKeys[i]; if (excluded.indexOf(key) >= 0) continue; if (!Object.prototype.propertyIsEnumerable.call(source, key)) continue; target[key] = source[key]; } } return target; }

function _objectWithoutPropertiesLoose(source, excluded) { if (source == null) return {}; var target = {}; var sourceKeys = Object.keys(source); var key, i; for (i = 0; i < sourceKeys.length; i++) { key = sourceKeys[i]; if (excluded.indexOf(key) >= 0) continue; target[key] = source[key]; } return target; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; var ownKeys = Object.keys(source); if (typeof Object.getOwnPropertySymbols === 'function') { ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) { return Object.getOwnPropertyDescriptor(source, sym).enumerable; })); } ownKeys.forEach(function (key) { _defineProperty(target, key, source[key]); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var setPercent = function setPercent(node, percentKey, nonPercentKey, fallbackKey, value) {
  if (value) {
    var parsedValue = parseFloat(value, 10);
    var isPercent = value.toString().indexOf("%") !== -1;

    if (isPercent) {
      node[percentKey](parsedValue);
    } else {
      node[nonPercentKey](parsedValue);
    }
  } else {
    node[fallbackKey]();
  }
};

var setNodePadding = function setNodePadding(node, val, dir) {
  var edge = (0, _yoga.getEdge)(dir);

  if (val) {
    if (val === "auto") {
      node.setPaddingAuto(edge);
    } else if (val.indexOf("%") !== -1) {
      node.setPaddingPercent(edge, parseFloat(val, 10));
    } else {
      node.setPadding(edge, val);
    }
  }
};

var setNodeMargin = function setNodeMargin(node, val, dir) {
  var edge = (0, _yoga.getEdge)(dir);

  if (val) {
    if (val === "auto") {
      node.setMarginAuto(edge);
    } else if (val.indexOf("%") !== -1) {
      node.setMarginPercent(edge, parseFloat(val, 10));
    } else {
      node.setMargin(edge, val);
    }
  }
};

var setStyles = function setStyles(node) {
  var style = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

  if (style.alignContent) {
    var alignContent = (0, _yoga.getAlign)(style.alignContent);
    node.setAlignContent(alignContent);
  }

  if (style.alignItems) {
    var alignItems = (0, _yoga.getAlign)(style.alignItems);
    node.setAlignItems(alignItems);
  }

  if (style.display) {
    var display = (0, _yoga.getDisplay)(style.display);
    node.setDisplay(display);
  }

  if (style.flexDirection) {
    var flexDirection = (0, _yoga.getFlexDirection)(style.flexDirection);
    node.setFlexDirection(flexDirection);
  }

  if (style.flexWrap) {
    var flexWrap = (0, _yoga.getFlexWrap)(style.flexWrap);
    node.setFlexWrap(flexWrap);
  }

  if (style.justifyContent) {
    var justifyContent = (0, _yoga.getJustifyContent)(style.justifyContent);
    node.setJustifyContent(justifyContent);
  }

  if (style.aspectRatio) {
    node.setAspectRatio(style.aspectRatio);
  }

  if (style.flex) {
    node.setFlex(style.flex);
  }

  if (style.flexGrow) {
    node.setFlexGrow(style.flexGrow);
  }

  if (style.flexShrink) {
    node.setFlexShrink(style.flexShrink);
  }

  if (style.flexBasis) {
    node.setFlexBasis(style.flexBasis);
  }

  if (style.flexBasisPercent) {
    node.setFlexBasisPercent(style.flexBasisPercent);
  } // Set Height


  if (style.minHeight) {
    node.setMinHeight(style.minHeight);
  }

  if (style.maxHeight) {
    node.setMaxHeight(style.maxHeight);
  }

  setPercent(node, "setHeightPercent", "setHeight", "setHeightAuto", style.height); // Set Width

  if (style.minWidth) {
    node.setMinWidth(style.minWidth);
  }

  if (style.maxWidth) {
    node.setMaxWidth(style.maxWidth);
  }

  setPercent(node, "setWidthPercent", "setWidth", "setWidthAuto", style.width); // Borders

  if (style.borderWidth) {
    node.setBorder((0, _yoga.getEdge)("all"), style.borderWidth);
  } // Margins


  setNodeMargin(node, style.marginTop, "top");
  setNodeMargin(node, style.marginRight, "right");
  setNodeMargin(node, style.marginBottom, "bottom");
  setNodeMargin(node, style.marginleft, "left");

  if (style.margin) {
    node.setMargin((0, _yoga.getEdge)("all"), style.margin);
  } // Padding


  setNodePadding(node, style.paddingTop, "top");
  setNodePadding(node, style.paddingRight, "right");
  setNodePadding(node, style.paddingBottom, "bottom");
  setNodePadding(node, style.paddingleft, "left");

  if (style.padding) {
    node.setPadding((0, _yoga.getEdge)("all"), style.padding);
  }

  return node;
};

var mergeStyles = function mergeStyles(_ref) {
  var style = _ref.style,
      width = _ref.width,
      height = _ref.height,
      strokeWidth = _ref.strokeWidth;
  return _objectSpread({}, style, {
    width: width,
    height: height,
    borderWidth: strokeWidth
  });
};

var Flexbox = function Flexbox(_ref2) {
  var children = _ref2.children,
      props = _objectWithoutProperties(_ref2, ["children"]);

  // Create Root Node
  var root = _yogaLayout.Node.create();

  var rootStyle = mergeStyles(props);
  root = setStyles(root, rootStyle); // Create Child Nodes & Add Styles

  _react.default.Children.forEach(children, function (c, i) {
    var child = _yogaLayout.Node.create();

    var style = mergeStyles(c.props);
    child = setStyles(child, style);
    root.insertChild(child, i);
    return child;
  }); // Calculate the Layout


  root.calculateLayout(rootStyle.width, rootStyle.height, _yogaLayout.default.DIRECTION_LTR);
  var rootLayout = root.getComputedLayout(); // Clone Children with new Layout props

  var newChildren = _react.default.Children.map(children, function (c, i) {
    var childNode = root.getChild(i);
    var layout = childNode.getComputedLayout();
    return _react.default.cloneElement(c, {
      width: layout.width,
      height: layout.height,
      transform: "translate(".concat(layout.left, " ").concat(layout.top, ")")
    });
  }); // Return new group with layout applied


  return _react.default.createElement("g", {
    width: rootLayout.width,
    height: rootLayout.height,
    transform: "translate(".concat(rootLayout.left, " ").concat(rootLayout.top, ")")
  }, newChildren);
};

exports.Flexbox = Flexbox;
Flexbox.propTypes = _propTypes.flexboxPropTypes;
Flexbox.defaultProps = {
  display: "flex",
  flex: 1
};