"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.measureText = exports.measureHeight = void 0;

var _unitsCss = _interopRequireDefault(require("units-css"));

var _canvas = require("canvas");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// Modified version of https://github.com/FormidableLabs/measure-text
// to run in node context.
var DEFAULT_CANVAS = (0, _canvas.createCanvas)();
var DEFAULT_FONT_WEIGHT = 400;
var DEFAULT_FONT_STYLE = "normal";

var measureHeight = function measureHeight(size, lineHeight) {
  // If the line-height is unitless,
  // multiply it by the font size.
  if (!lineHeight.unit) {
    return _unitsCss.default.parse("".concat(size.value * lineHeight.value).concat(size.unit));
  } // units-css requires the user to provide
  // DOM nodes for these units. We don't want
  // to pollute our API with that for the time being.


  var unitBlacklist = ["%", "ch", "cm", "em", "ex"];

  if (unitBlacklist.indexOf(lineHeight.unit) !== -1) {
    // eslint-disable-line no-magic-numbers
    throw new Error("We do not currently support the unit ".concat(lineHeight.unit, "\n      from the provided line-height ").concat(lineHeight.value, ".\n      Unsupported units include ").concat(unitBlacklist.join(", "), "."));
  } // Otherwise, the height is equivalent
  // to the provided line height.
  // Non-px units need conversion.


  if (lineHeight.unit === "px") {
    return lineHeight;
  }

  return _unitsCss.default.parse(_unitsCss.default.convert(lineHeight, "px"));
};

exports.measureHeight = measureHeight;

var measureText = function measureText(_ref) {
  var text = _ref.text,
      fontFamily = _ref.fontFamily,
      fontSize = _ref.fontSize,
      lineHeight = _ref.lineHeight,
      _ref$fontWeight = _ref.fontWeight,
      fontWeight = _ref$fontWeight === void 0 ? DEFAULT_FONT_WEIGHT : _ref$fontWeight,
      _ref$fontStyle = _ref.fontStyle,
      fontStyle = _ref$fontStyle === void 0 ? DEFAULT_FONT_STYLE : _ref$fontStyle,
      _ref$canvas = _ref.canvas,
      canvas = _ref$canvas === void 0 ? DEFAULT_CANVAS : _ref$canvas;
  var ctx = canvas.getContext("2d");
  ctx.font = "".concat(fontWeight, " ").concat(fontStyle, " ").concat(fontSize, " ").concat(fontFamily);

  var measure = function measure(line) {
    return {
      text: line,
      width: _unitsCss.default.parse("".concat(ctx.measureText(line).width, "px")),
      height: measureHeight(_unitsCss.default.parse(fontSize), _unitsCss.default.parse(lineHeight))
    };
  }; // If multiline, measure the bounds
  // of all of the lines combined


  if (Array.isArray(text)) {
    return text.map(measure).reduce(function (prev, curr) {
      var width = curr.width.value > prev.width.value ? curr.width : prev.width;

      var height = _unitsCss.default.parse("".concat(prev.height.value + curr.height.value).concat(curr.height.unit));

      var longest = curr.text.length > prev.text.length ? curr.text : prev.text;
      return {
        width: width,
        height: height,
        text: longest
      };
    });
  }

  return measure(text);
};

exports.measureText = measureText;