"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.splitText = void 0;

var _measuretext = require("./measuretext");

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; var ownKeys = Object.keys(source); if (typeof Object.getOwnPropertySymbols === 'function') { ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) { return Object.getOwnPropertyDescriptor(source, sym).enumerable; })); } ownKeys.forEach(function (key) { _defineProperty(target, key, source[key]); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

// Sourced from https://codereview.stackexchange.com/a/16112
// Modified to run in node.
var splitText = function splitText(text, maxWidth) {
  var fontStyle = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};
  var emmeasure = (0, _measuretext.measureText)(_objectSpread({
    text: "M"
  }, fontStyle)).width.value;
  var spacemeasure = (0, _measuretext.measureText)(_objectSpread({
    text: " "
  }, fontStyle)).width.value; // To prevent weird looping anamolies farther on.

  if (maxWidth < emmeasure) {
    throw "Can't fragment less than one character.";
  }

  if ((0, _measuretext.measureText)(_objectSpread({
    text: text
  }, fontStyle)).width.value < maxWidth) {
    return [text];
  }

  var words = text.split(" "),
      metawords = [],
      lines = []; // measure first.

  for (var w in words) {
    var word = words[w];
    var measure = (0, _measuretext.measureText)(_objectSpread({
      text: word
    }, fontStyle)).width.value; // Edge case - If the current word is too long for one line, break it into maximized pieces.

    if (measure > maxWidth) {
      // TODO - a divide and conquer method might be nicer.
      var edgewords = function (word, maxWidth) {
        var wlen = word.length;
        if (wlen == 0) return [];
        if (wlen == 1) return [word];
        var awords = [],
            cword = "",
            cmeasure = 0,
            letters = []; // Measure each letter.

        for (var l = 0; l < wlen; l++) {
          letters.push({
            letter: word[l],
            measure: (0, _measuretext.measureText)(_objectSpread({
              text: word[l]
            }, fontStyle)).width.value
          });
        } // Assemble the letters into words of maximized length.


        for (var ml in letters) {
          var metaletter = letters[ml];

          if (cmeasure + metaletter.measure > maxWidth) {
            awords.push({
              word: cword,
              len: cword.length,
              measure: cmeasure
            });
            cword = "";
            cmeasure = 0;
          }

          cword += metaletter.letter;
          cmeasure += metaletter.measure;
        } // there will always be one more word to push.


        awords.push({
          word: cword,
          len: cword.length,
          measure: cmeasure
        });
        return awords;
      }(word, maxWidth); // could use metawords = metawords.concat(edgwords)


      for (var ew in edgewords) {
        metawords.push(edgewords[ew]);
      }
    } else {
      metawords.push({
        word: word,
        len: word.length,
        measure: measure
      });
    }
  } // build array of lines second.


  var cline = "";
  var cmeasure = 0;

  for (var mw in metawords) {
    var metaword = metawords[mw]; // If current word doesn't fit on current line, push the current line and start a new one.
    // Unless (edge-case): this is a new line and the current word is one character.

    if (cmeasure + metaword.measure > maxWidth && cmeasure > 0 && metaword.len > 1) {
      lines.push(cline);
      cline = "";
      cmeasure = 0;
    }

    cline += metaword.word;
    cmeasure += metaword.measure; // If there's room, append a space, else push the current line and start a new one.

    if (cmeasure + spacemeasure < maxWidth) {
      cline += " ";
      cmeasure += spacemeasure;
    } else {
      lines.push(cline);
      cline = "";
      cmeasure = 0;
    }
  }

  if (cmeasure > 0) lines.push(cline);
  return lines;
};

exports.splitText = splitText;